/*
 * File:  midi_tester_defs.h
 *
 * Definitions and declarations for the MIDI Device Tester application
 * 
 */ 
#ifndef  MIDI_TESTER_DEFS_H
#define  MIDI_TESTER_DEFS_H
 
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include "Pro_Micro_periph_lib.h"
#include "OLED_display_avr8_lib.h"
#include "MIDI_comms_lib.h"

#ifndef F_CPU
#define F_CPU  16000000UL    // CPU clock frequency
#endif

// Hard-coded constants -- may be changed only by re-compiling the code!
#define MIDI_OUT_CHANNEL  1     // 1..16
#define KEY_VELOCITY      80    // 

// Push-buttons 'A', 'B', 'C', 'D' are wired to port pins PB1, PB2, PB3, PB4, resp.
// Macro to configure switch input pins, enable internal pull-ups:
#define SETUP_SWITCH_INPUT_PINS()  { DDRB &= 0b11100001;  PORTB |= 0b00011110; }
    
#define BUTTON_A_PRESSED  ((PINB & (1<<1)) == 0)
#define BUTTON_B_PRESSED  ((PINB & (1<<2)) == 0)
#define BUTTON_C_PRESSED  ((PINB & (1<<3)) == 0)
#define BUTTON_D_PRESSED  ((PINB & (1<<4)) == 0)

#define LEFT_KEY    0     // Touch keypad identifiers
#define CENTRE_KEY  1
#define RIGHT_KEY   2

#define LED1_TURN_ON()    SET_BIT(PORTC, 6)
#define LED1_TURN_OFF()   CLEAR_BIT(PORTC, 6)
#define LED2_TURN_ON()    SET_BIT(PORTD, 7)
#define LED2_TURN_OFF()   CLEAR_BIT(PORTD, 7)
#define LED3_TURN_ON()    SET_BIT(PORTE, 6)
#define LED3_TURN_OFF()   CLEAR_BIT(PORTE, 6)

// Valid Note ON/OFF/Repeat states for 'PlayerTask' state-machine:
#define NOTE_OFF   0
#define NOTE_ON    1
#define NOTE_RPT   2

// Valid states for 'keypadState' state-machine...
#define KEYS_OFF_OFF_OFF   0x00
#define KEYS_OFF_OFF_ON    0x01
#define KEYS_OFF_ON_OFF    0x02
#define KEYS_OFF_ON_ON     0x03
#define KEYS_ON_OFF_OFF    0x04
#define KEYS_ON_OFF_ON     0x05
#define KEYS_ON_ON_OFF     0x06
#define KEYS_ON_ON_ON      0x07

// Note sequence "direction" may be any one of these:
#define FIXED_REF         0x00    // Next note must be the reference note
#define SAME_AGAIN        0x03    // Next note must be same as last keyed
#define ANY_DIRN          0x07    // Next note may be higher, lower or same
#define GO_UP_3           0x1F    // Next note should be 3 semitones higher
#define GO_UP_6           0x7F    // Next note should be 6 semitones higher
#define GO_DOWN_3         0xEF    // Next note should be 3 semitones lower
#define GO_DOWN_6         0xBF    // Next note should be 6 semitones lower

#define DoNothing()       {}      // For code readability, etc


// ------------  Function prototype definitions  ------------------------------
void   PlayerTask();
void   MidiControlChange();
void   InitiateNote(uint8 dirn);
void   TerminateNote();
uint8  GetCurrentScreenID();
void   GoToNextScreen(uint8 screenID);
void   UserInterfaceTask();
uint8  NoteSequence(uint8 direction);
unsigned  FastRand(unsigned long seed);
uint8  GaussRand(uint8 minval, uint8 maxval, uint8 spread);
uint8  FilteRand(uint8 minval, uint8 maxval, uint8 klag);
void   ButtonScan(unsigned char nb);
bool   ButtonHit(char  button_ID);
void   PotService();
bool   PotMoved();
uint8  PotPosition();
void   KeypadScan();
bool   KeyTouched(uint8 pad);
bool   KeyStruck(uint8 pad);
bool   KeyReleased(uint8 pad);
void   KeypadClear();
void   TouchCalibrate();
void   TouchService();


// ------------  Global variables and data  -----------------------------------
//
extern unsigned  task1Freq;            // Call frequency of 1ms task (Hz)
extern uint8  midi_chan;               // MIDI channel # (1..16)
extern uint8  touch_onoff_thres[4];    // touch-pad on/off threshold values
extern uint8  touch_reading[4];        // touch-pad ADC readings (0..255)

extern uint8  exprnCCnum;         // MIDI expression CC number (typ. 2, 7 or 11)
extern uint8  referenceNote;      // MIDI note number (fixed)
extern uint8  notePlaying;        // MIDI note number (0 => No note playing)
extern uint8  velocity;           // MIDI velocity value (simulated)
extern uint8  modulation;         // MIDI modulation value (simulated)
extern uint8  expression;         // MIDI expression value (simulated)
extern uint8  keypadState;        // Keypad state (pads touched)


#endif  // MIDI_TESTER_DEFS_H
